unit FIB_SQLQuery;

interface
uses
  Classes, SysUtils, Variants, Dialogs,
  DB, CommonStuff,
  FIB_messages, FIB_header, FIB_array, FIB_Blob,
  FIB_params, FIB_access;

type

  TFIB_SQLStatement = class(TSQLObject)
  private
  protected
    FHandleError: Boolean;
    procedure Loaded; override;
    procedure SetDatabase(Value: TFIB_Database); virtual;
    procedure ValidateInsert(AComponent: TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function CanPrepareStatement(ForceDBConnect: Boolean): Boolean;
    procedure DBConnectChanged(Sender: TObject; Connecting: Boolean); virtual;
    function HandleStatementType: Boolean; virtual; abstract;
    function GetIsPrepared: Boolean; override;
    procedure UnPrepareStmt; override;
    procedure FinalizeExecute(CanCommit: Boolean); virtual;
    property PXSQLDA_IN: PXSQLDA read FPXSQLDA_IN;
    property SQL: TStrings read FSQLStmt write SetSQLStmt;
    property Params: TFIB_Params read FParams write SetParams;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property RowsAffected: Integer read FAffectedRows;
  published
    property AutoDBConnect: Boolean read FAutoDBConnect write FAutoDBConnect default True;
    property Database: TFIB_Database read FDatabase write SetDatabase;
    property Transaction: TFIB_Transaction read FTransaction write SetTransaction;
  end;

  TFIB_CreateAlterDrop = class(TFIB_SQLStatement)
  private
  protected
    function HandleStatementType: Boolean; override;
  public
    function ExecSQL(CanAutoCommit: Boolean = False): Boolean;
  published
    property SQL;
  end;


  TFIB_UpdateSQL = class(TFIB_SQLStatement)
  private
  protected
    function HandleStatementType: Boolean; override;
  public
    function ExecSQL(CanAutoCommit: Boolean = True): Boolean;
  published
    property SQL;
    property Params;
  end;

  TFIB_CustomSQL = class;

  TFIB_Field = class(TFIB_FieldParam)
  private
  protected
    function InternalGetMemoryStream: TMemoryStream; override;
  public
    constructor Create(AOwner: TFIB_FieldsParams); override;
    function IsBlob: Boolean;
    property FieldKind: TFieldKind read FFieldKind;
    property FieldName: string read FFieldName;
    property Size: Word read FSize;
  end;

  TFIB_Fields = class(TFIB_FieldsParams)
  private
  protected
    function InternalCreateItem: TFIB_FieldParam; override;
    function GetItemsCount: Word; override;
    procedure Clear;
  public
    constructor Create(POwnerDB: PFIB_Database); override;
    destructor Destroy; override;
    function FindField(FieldName: string): TFIB_Field;
    function FieldByName(FieldName: string): TFIB_Field;
    property Fields[Index: SmallInt]: TFIB_Field read GetItem; default;
  end;

  TFIB_CustomSQL = class(TFIB_SQLStatement)
  private
  protected
    function HandleStatementType: Boolean; override;
    procedure FinalizeExecute(CanCommit: Boolean); override;
    procedure UnPrepareStmt; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Fields: TFIB_Fields read FFields;
    property FieldCount: Word read GetFieldCount;
  published
    property Params;
  end;

  TFIB_Query = class(TFIB_CustomSQL)
  private
  protected
  public
    function ExecSQL(CanAutoCommit: Boolean = False): Boolean;
    function FetchRecord: Boolean;
  published
    property FetchRecordOnExec: Boolean read FFetchRecordOnExec write FFetchRecordOnExec;
    property SQL;
  end;

  TFIB_StoredProc = class(TFIB_CustomSQL)
  private
  protected
    procedure SetDatabase(Value: TFIB_Database); override;
    procedure DBConnectChanged(Sender: TObject; Connecting: Boolean); override;
  public
    function ExecProc(CanAutoCommit: Boolean = True): Boolean;
  published
    property ProcName: string read FProcName write SetProcName;
  end;


  TFIB_Generator = class(TFIB_SQLStatement)
  private
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CreateGenerator(AGeneratorName: string; MakeItCurrent: Boolean): Boolean;
    function DropGenerator(AGeneratorName: string): Boolean;
    property CurrentValue: Int64 read GetCurrentValue write SetCurrentValue;
    property NextValue: Int64 read IncrementGenerator;
  published
    property GeneratorName: string read FGeneratorName write SetGeneratorName;
    property Increment: SmallInt read FIncrement write FIncrement default 1;
  end;


implementation

end.

