unit FIB_access;

interface
{$I Version.inc}
uses
  Classes, SysUtils, DB, Dialogs,
  FIB_header, FIB_messages;

type
  TTransactionAction = (taCommit, taCommitRetaining, taRollback, taRollbackRetaining);
  THandlingParamsChangeWhenActive = (paAllowCommit, paAllowRollback,
                                     paNotAllowSilent, paRaiseError);

  TEndTransactionEvent = procedure(Sender: TObject; Action: TTransactionAction) of object;

  TDataBaseInfo = packed
  record
    ODSMajorVersion: SmallInt;
    ODSMinorVersion: SmallInt;
    FBServerVersion: string;
    FBDefaultCharSet: string;
  end;

  TFIB_Transaction = class;
  PFIB_Database = ^TFIB_Database;
  TFIB_Database = class(TCustomConnection)
  private
  protected
    Ffbclib: TFIBClientLibrary;
    FAssignmentChanged: Boolean;
    function CheckFclibLoaded: Boolean;
    procedure GetFIBSysTabValues(SQLstring: string; SysTabValues: TStrings);
    procedure GetAvailableCollations(Collations: TStrings);
    procedure GetStoredProcedureNames(OnlyWithOutParams: Boolean; List: TStrings);
    procedure GetGeneratorsNames(List: TStrings);
    procedure DoConnect; override;
    procedure DoDisconnect; override;
    function GetConnected: Boolean; override;
    procedure AddDelTransaction(FIB_Trans: TFIB_Transaction;
                                AddTransaction: Boolean);
    property DefaultTransaction: TFIB_Transaction read GetDefaultTransaction;
    function LoginParamsAssigned: Boolean;
    property HandleIsShared: Boolean read FHandleIsShared;
    property TableNames: TStrings read GetTableNames;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetHandle(Value: TISC_DB_HANDLE);
    procedure AddDelClient(Client: TObject; AddClient: Boolean;
                           Event: TConnectChangeEvent = nil);
    property Handle: TISC_DB_HANDLE read FHandle;
    property DataBaseInfo: TDataBaseInfo read FDataBaseInfo;
  published
    property ClientLibraryPath: string read FClientLibraryPath write SetClientLibraryPath;
    property ClientLibrary: TClientLibrary read FClientLibraryType write SetClientLibrary
             default clAutoDetect;
    property IsInterbaseDB: Boolean read FisInterbaseDB;
    property DatabaseName: string read FDBName write SetDatabaseName;
    property Params: TStrings read FDBParams write SetDBParams;
    property SQLDialect: Integer read FSQLDialect write SetSQLDialect;
    property SQLRoleName: string read FSQLRoleName write SetSQLRoleName;
    property UserName: string read FUserName write SetUserName;
    property Password: string read FPassword write SetPassword;
    property CharacterSet: string read FCharacterSet write SetCharacterSet;
    property Collation: string read FCollation write FCollation;
    property Connected;
    property LoginPrompt;
    property AfterConnect;
    property BeforeConnect;
    property AfterDisconnect;
    property BeforeDisconnect;
    property OnLogin;
  end;

  TFIB_Transaction = class (TComponent)
  private
  protected
    procedure SetHandle(Value: TISC_TR_HANDLE);
    procedure AddDataSetToTransaction(Dataset: TDataset);
    procedure EndTransaction(Action: TTransactionAction; Restart: Boolean);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    property HandleIsShared: Boolean read FHandleIsShared;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure StartTransaction;
    procedure Commit;
    procedure CommitRetaining;
    procedure Rollback;
    procedure RollbackRetaining;
    procedure ApplyDefaultAction;
    property Handle: TISC_TR_HANDLE read FHandle;
    property Active: Boolean read IsStarted write SetActive;
    property InProgress: Boolean read FInProgress;
  published
    property AutoCommit: Boolean read FAutoCommit write SetAutoCommit default True;
    property DefaultAction: TTransactionAction read FDefaultAction
             write FDefaultAction{SetDefaultAction} default taCommitRetaining;
    property Database: TFIB_Database read FDatabase
             write SetDatabase;
    property Params: TStrings read FTRParams write SetTRParams;
    property ParamsChangeAction: THandlingParamsChangeWhenActive
             read FParamsChangeAction write FParamsChangeAction
             default paNotAllowSilent;
    property AfterEnd: TEndTransactionEvent read FAfterEnd write FAfterEnd;
    property BeforeEnd: TEndTransactionEvent read FBeforeEnd write FBeforeEnd;
  end;


implementation

end.
