unit FIB_header;

interface
{$I Version.Inc}
uses
  Windows, Classes, SysUtils, DateUtils, Dialogs, Variants, Math,
  FIB_messages;

type

  TSQLObject = class(TComponent)
  protected
    function GetIsPrepared: Boolean; virtual; abstract;
    procedure UnPrepareStmt; virtual; abstract;    
  public
    property IsPrepared: Boolean read GetIsPrepared;
  end;

  { C Date/Time Structure }
  TCTimeStructure = record
    tm_sec: integer;   { Seconds }
    tm_min: integer;   { Minutes }
    tm_hour: integer;  { Hour (0--23) }
    tm_mday: integer;  { Day of month (1--31) }
    tm_mon: integer;   { Month (0--11) }
    tm_year: integer;  { Year (calendar year minus 1900) }
    tm_wday: integer;  { Weekday (0--6) Sunday = 0) }
    tm_yday: integer;  { Day of year (0--365) }
    tm_isdst: integer; { 0 if daylight savings time is not in effect) }
  end;
  PCTimeStructure = ^TCTimeStructure;  

  Float  = Single;  { 32 bit real value }
  PFloat = ^Float;

  Void = Pointer;
  PVoid = ^Void;

  TISC_DB_HANDLE = PVoid;
  PISC_DB_HANDLE = ^TISC_DB_HANDLE;

  TISC_TR_HANDLE = PVoid;
  PISC_TR_HANDLE = ^TISC_TR_HANDLE;

  TISC_STMT_HANDLE = PVoid;
  PISC_STMT_HANDLE = ^TISC_STMT_HANDLE;

  TISC_BLOB_HANDLE = PVoid;
  PISC_BLOB_HANDLE = ^TISC_BLOB_HANDLE;

  ISC_STATUS     = LongInt;
  PISC_STATUS    = ^ISC_STATUS;
  PPISC_STATUS   = ^PISC_STATUS;

  TStatusVector  = array[0..19] of ISC_STATUS;
  PStatusVector  = ^TStatusVector;

//
  (****************************************************************)
  (** record type is for passing arguments to isc_start_multiple **)
  (****************************************************************)
  TISC_TEB = record
    db_handle      : PISC_DB_HANDLE;
    tpb_length     : LongInt;
    tpb_address    : PChar;
  end;
  PISC_TEB = ^TISC_TEB;


  (********************************)
  (** Declare the extended SQLDA **)
  (********************************)
  TXSQLVAR = record
    sqltype                     : SmallInt;     (** datatype of field **)
    sqlscale                    : SmallInt;     (** scale factor **)
    sqlsubtype                  : SmallInt;     (** datatype subtype - BLOBs **)
               (** & text types only **)
    sqllen                      : SmallInt;     (** length of data area **)
    sqldata                     : PChar;        (** address of data **)
    sqlind                      : PSmallInt;    (** address of indicator **)
                                                (** variable **)
    sqlname_length              : SmallInt;     (** length of sqlname field **)
    (** name of field, name length + space for NULL **)
    sqlname                     : array[0..31] of Char;
    relname_length              : SmallInt;     (** length of relation name **)
    (** field's relation name + space for NULL **)
    relname                     : array[0..31] of Char;
    ownname_length              : SmallInt;     (** length of owner name **)
    (** relation's owner name + space for NULL **)
    ownname                     : array[0..31] of Char;
    aliasname_length            : SmallInt;     (** length of alias name **)
    (** relation's alias name + space for NULL **)
    aliasname                   : array[0..31] of Char;
  end;  // TXSQLVAR
  PXSQLVAR                      = ^TXSQLVAR;

  TXSQLDA = record
    version: SmallInt;                 { version of this XSQLDA }
    sqldaid: array[0..7] of char;     { XSQLDA name field }
    sqldabc: Integer;                  { length in bytes of SQLDA }
    sqln:    SmallInt;                 { number of fields allocated }
    sqld:    SmallInt;                 { actual number of fields }
    sqlvar:  array[0..0] of TXSQLVAR; { first field address }
  end; // TXSQLDA
  PXSQLDA = ^TXSQLDA;

  { Time & Date Support }
  ISC_DATE = Integer;
  PISC_DATE = ^ISC_DATE;
  ISC_TIME = Cardinal;
  PISC_TIME = ^ISC_TIME;

  TISC_TIMESTAMP = record
    timestamp_date: ISC_DATE;
    timestamp_time: ISC_TIME;
  end;
  PISC_TIMESTAMP = ^TISC_TIMESTAMP;


  { Blob id structure }
  TGDS_QUAD = record
    gds_quad_high: Integer;
    gds_quad_low: Cardinal;
  end;
  PGDS_QUAD = ^TGDS_QUAD;

  { array structures }
  TISC_ARRAY_BOUND = record
    array_bound_lower: SmallInt;
    array_bound_upper: SmallInt;
  end;
  PISC_ARRAY_BOUND     = ^TISC_ARRAY_BOUND;

  TISC_ARRAY_DESC = record
    array_desc_dtype:         Byte;
    array_desc_scale:         ShortInt;
    array_desc_length:        Word;
    array_desc_field_name:    array[0..31] of Char;
    array_desc_relation_name: array[0..31] of Char;
    array_desc_dimensions:    SmallInt;
    array_desc_flags:         SmallInt;
    array_desc_bounds:        array[0..15] of TISC_ARRAY_BOUND;
  end;
  PISC_ARRAY_DESC = ^TISC_ARRAY_DESC;


  Tisc_get_client_version = procedure(CliVerStr            : PChar);
                                      stdcall;

  Tisc_attach_database = function(status_vector            : PISC_STATUS;
                                  db_name_length           : SmallInt;
                                  db_name                  : PChar;
                                  db_handle                : PISC_DB_HANDLE;
                                  parm_buffer_length       : SmallInt;
                                  parm_buffer              : PChar): ISC_STATUS;
                                  stdcall;

  Tisc_detach_database = function(status_vector            : PISC_STATUS;
                                  db_handle                : PISC_DB_HANDLE): ISC_STATUS;
                                  stdcall;

  Tisc_database_info = function(status_vector              : PISC_STATUS;
                                db_handle                  : PISC_DB_HANDLE;
                                item_list_buffer_length    : Short;
                                item_list_buffer           : PChar;
                                result_buffer_length       : Short;
                                result_buffer              : PChar): ISC_STATUS;
                                stdcall;

  Tisc_start_transaction = function(status_vector          : PISC_STATUS;
                                    tran_handle            : PISC_TR_HANDLE;
                                    db_handle_count        : SmallInt;
                                    db_handle              : PISC_DB_HANDLE;
                                    tpb_length             : Word;
                                    tpb_address            : PChar): ISC_STATUS;
                                    cdecl;

  Tisc_start_multiple = function(status_vector             : PISC_STATUS;
                                 tran_handle               : PISC_TR_HANDLE;
                                 db_handle_count           : SmallInt;
                                 teb_vector_address        : PISC_TEB): ISC_STATUS;
                                 stdcall;

  Tisc_commit_retaining = function(status_vector           : PISC_STATUS;
                                   tran_handle             : PISC_TR_HANDLE): ISC_STATUS;
                                   stdcall;

  Tisc_commit_transaction = function(status_vector         : PISC_STATUS;
                                     tran_handle           : PISC_TR_HANDLE): ISC_STATUS;
                                     stdcall;

  Tisc_rollback_transaction = function(status_vector       : PISC_STATUS;
                                       tran_handle         : PISC_TR_HANDLE): ISC_STATUS;
                                       stdcall;

  Tisc_rollback_retaining = function(status_vector         : PISC_STATUS;
                                     tran_handle           : PISC_TR_HANDLE): ISC_STATUS;
                                     stdcall;

  Tisc_dsql_execute_immediate = function(status_vector     : PISC_STATUS;
                                         db_handle         : PISC_DB_HANDLE;
                                         tran_handle       : PISC_TR_HANDLE;
                                         length            : Word;
                                         statement         : PChar;
                                         dialect           : Word;
                                         xsqlda            : PXSQLDA): ISC_STATUS;
                                         stdcall;

  Tisc_dsql_exec_immed2 = function(status_vector           : PISC_STATUS;
                                   db_handle               : PISC_DB_HANDLE;
                                   tran_handle             : PISC_TR_HANDLE;
                                   length                  : Word;
                                   statement               : PChar;
                                   dialect                 : Word;
                                   in_xsqlda,
                                   out_xsqlda              : PXSQLDA): ISC_STATUS;
                                   stdcall;

  Tisc_dsql_allocate_statement = function(status_vector    : PISC_STATUS;
                                          db_handle        : PISC_DB_HANDLE;
                                          stmt_handle      : PISC_STMT_HANDLE): ISC_STATUS;
                                          stdcall;

  Tisc_dsql_alloc_statement2 = function(status_vector      : PISC_STATUS;
                                        db_handle          : PISC_DB_HANDLE;
                                        stmt_handle        : PISC_STMT_HANDLE): ISC_STATUS;
                                        stdcall;

  Tisc_dsql_free_statement = function(status_vector        : PISC_STATUS;
                                      stmt_handle          : PISC_STMT_HANDLE;
                                      options              : Word): ISC_STATUS;
                                      stdcall;

  Tisc_dsql_prepare = function(status_vector               : PISC_STATUS;
                               tran_handle                 : PISC_TR_HANDLE;
                               stmt_handle                 : PISC_STMT_HANDLE;
                               length                      : Word;
                               statement                   : PChar;
                               dialect                     : Word;
                               xsqlda                      : PXSQLDA): ISC_STATUS;
                               stdcall;

  Tisc_dsql_describe_bind = function(status_vector         : PISC_STATUS;
                                     stmt_handle           : PISC_STMT_HANDLE;
                                     dialect               : Word;
                                     xsqlda                : PXSQLDA): ISC_STATUS;
                                     stdcall;

  Tisc_dsql_describe = function(status_vector              : PISC_STATUS;
                                stmt_handle                : PISC_STMT_HANDLE;
                                dialect                    : Word;
                                xsqlda                     : PXSQLDA): ISC_STATUS;
                                stdcall;

  Tisc_dsql_execute = function(status_vector               : PISC_STATUS;
                               tran_handle                 : PISC_TR_HANDLE;
                               stmt_handle                 : PISC_STMT_HANDLE;
                               dialect                     : Word;
                               xsqlda                      : PXSQLDA): ISC_STATUS;
                               stdcall;

  Tisc_dsql_execute2 = function(status_vector              : PISC_STATUS;
                                tran_handle                : PISC_TR_HANDLE;
                                stmt_handle                : PISC_STMT_HANDLE;
                                dialect                    : Word;
                                in_xsqlda,
                                out_xsqlda                 : PXSQLDA): ISC_STATUS;
                                stdcall;

  Tisc_dsql_sql_info = function(status_vector              : PISC_STATUS;
                                stmt_handle                : PISC_STMT_HANDLE;
                                item_length                : SmallInt;
                                items                      : PChar;
                                buffer_length              : SmallInt;
                                buffer                     : PChar): ISC_STATUS;
                                stdcall;

  Tisc_dsql_fetch = function(status_vector                 : PISC_STATUS;
                             stmt_handle                   : PISC_STMT_HANDLE;
                             dialect                       : Word;
                             xsqlda                        : PXSQLDA): ISC_STATUS;
                             stdcall;

  Tisc_vax_integer = function(buffer: PChar; length: SmallInt): ISC_STATUS;
                              stdcall;

  Tisc_sqlcode = function(status_vector: PISC_STATUS): Integer; stdcall;
  Tisc_interprete = function(buffer: PChar;
                             status_vector: PPISC_STATUS): ISC_STATUS;
                             stdcall;

  Tisc_decode_sql_time = procedure(ib_time: PISC_TIME;
                                   tm_date: PCTimeStructure);
                                   stdcall;

  Tisc_decode_sql_date = procedure(ib_date: PISC_DATE;
                                   tm_date: PCTimeStructure);
                                   stdcall;

  Tisc_decode_timestamp = procedure(ib_timestamp: PISC_TIMESTAMP;
                                    tm_date: PCTimeStructure);
                                    stdcall;

  Tisc_encode_sql_date = procedure(tm_date: PCTimeStructure;
                                   ib_date: PISC_DATE);
                                   stdcall;

  Tisc_encode_sql_time = procedure(tm_date: PCTimeStructure;
                                   ib_time: PISC_TIME);
                                   stdcall;

  Tisc_encode_timestamp = procedure(tm_date: PCTimeStructure;
                                    ib_timestamp: PISC_TIMESTAMP);
                                    stdcall;

  Tisc_array_lookup_bounds = function(status_vector: PISC_STATUS;
                                      db_handle: PISC_DB_HANDLE;
                                      trans_handle: PISC_TR_HANDLE;
                                      table_name, column_name: PChar;
				      array_descriptor: PISC_ARRAY_DESC): ISC_STATUS;
                                      stdcall;

  Tisc_array_get_slice = function(status_vector: PISC_STATUS;
                                  db_handle:     PISC_DB_HANDLE;
                                  trans_handle:  PISC_TR_HANDLE;
                                  array_id:      PGDS_QUAD;
				  array_descriptor: PISC_ARRAY_DESC;
                                  dest_array:    PVoid;
				  slice_length:  PInteger): ISC_STATUS;
                                  stdcall;

  Tisc_array_put_slice = function(status_vector: PISC_STATUS;
                                  db_handle:     PISC_DB_HANDLE;
                                  trans_handle:  PISC_TR_HANDLE;
                                  array_id:      PGDS_QUAD;
                                  array_descriptor: PISC_ARRAY_DESC;
                                  source_array:  PVoid;
                                  slice_length:  PInteger): ISC_STATUS;
                                  stdcall;

  Tisc_blob_info = function(status_vector: PISC_STATUS;
                            blob_handle: PISC_BLOB_HANDLE;
                            item_list_buffer_length: Short;
                            item_list_buffer: PChar;
                            result_buffer_length: Short;
                            result_buffer: PChar): ISC_STATUS;
                            stdcall;

  Tisc_create_blob = function(status_vector: PISC_STATUS;
                              db_handle: PISC_DB_HANDLE;
                              tran_handle: PISC_TR_HANDLE;
                              blob_handle: PISC_BLOB_HANDLE;
                              blob_id: PGDS_QUAD): ISC_STATUS;
                              stdcall;

  Tisc_open_blob2 = function(status_vector: PISC_STATUS;
                             db_handle: PISC_DB_HANDLE;
                             tran_handle: PISC_TR_HANDLE;
                             blob_handle: PISC_BLOB_HANDLE;
                             blob_id: PGDS_QUAD;
                             bpb_length: Smallint;
                             bpb_buffer: PChar): ISC_STATUS;
                             stdcall;

  Tisc_get_segment = function(status_vector: PISC_STATUS;
                              blob_handle: PISC_BLOB_HANDLE;
                              actual_seg_length: PWord;
                              seg_buffer_length: Word;
                              seg_buffer: PChar): ISC_STATUS;
                              stdcall;

  Tisc_put_segment = function(status_vector: PISC_STATUS;
                              blob_handle: PISC_BLOB_HANDLE;
                              seg_buffer_len: Word;
                              seg_buffer: PChar): ISC_STATUS;
                              stdcall;

  Tisc_close_blob = function(status_vector: PISC_STATUS;
                             blob_handle: PISC_BLOB_HANDLE): ISC_STATUS;
                             stdcall;


  EFBLError = class(Exception)
  private
  protected
    property SQLCode: Integer read FSQLCode write FSQLCode;
    property ISC_ErrorCode: Integer read FError_Code;
  public
    constructor Create(ASQLCode, AErrorCode: Integer; AMsg: string); overload;
    constructor Create(AMsg: string); overload;
  end;

  TClientLibrary = (clAutoDetect, clFBEmbed, clFBClient, clGDS32);

  TFIBClientLibrary = class(TObject)
  private
  public
    isc_get_client_version:      Tisc_get_client_version;
    isc_attach_database:         Tisc_attach_database;
    isc_detach_database:         Tisc_detach_database;
    isc_database_info:           Tisc_database_info;
    isc_start_transaction:       Tisc_start_transaction;
    isc_start_multiple:          Tisc_start_multiple;
    isc_commit_retaining:        Tisc_commit_retaining;
    isc_commit_transaction:      Tisc_commit_transaction;
    isc_rollback_transaction:    Tisc_rollback_transaction;
    isc_rollback_retaining:      Tisc_rollback_retaining;
    isc_dsql_execute_immediate:  Tisc_dsql_execute_immediate;
    isc_dsql_exec_immed2:        Tisc_dsql_exec_immed2;
    isc_dsql_allocate_statement: Tisc_dsql_allocate_statement;
    isc_dsql_alloc_statement2:   Tisc_dsql_alloc_statement2;
    isc_dsql_free_statement:     Tisc_dsql_free_statement;
    isc_dsql_prepare:            Tisc_dsql_prepare;
    isc_dsql_describe_bind:      Tisc_dsql_describe_bind;
    isc_dsql_describe:           Tisc_dsql_describe;
    isc_dsql_execute:            Tisc_dsql_execute;
    isc_dsql_execute2:           Tisc_dsql_execute2;
    isc_dsql_sql_info:           Tisc_dsql_sql_info;
    isc_dsql_fetch:              Tisc_dsql_fetch;
    isc_vax_integer:             Tisc_vax_integer;
    isc_sqlcode:                 Tisc_sqlcode;
    isc_interprete:              Tisc_interprete;
    isc_decode_sql_time:         Tisc_decode_sql_time;
    isc_decode_sql_date:         Tisc_decode_sql_date;
    isc_decode_timestamp:        Tisc_decode_timestamp;
    isc_encode_sql_date:         Tisc_encode_sql_date;
    isc_encode_sql_time:         Tisc_encode_sql_time;
    isc_encode_timestamp:        Tisc_encode_timestamp;

    isc_array_lookup_bounds:     Tisc_array_lookup_bounds;
    isc_array_get_slice:         Tisc_array_get_slice;
    isc_array_put_slice:         Tisc_array_put_slice;

    isc_blob_info:               Tisc_blob_info;
    isc_create_blob:             Tisc_create_blob;
    isc_open_blob2:              Tisc_open_blob2;
    isc_get_segment:             Tisc_get_segment;
    isc_put_segment:             Tisc_put_segment;
    isc_close_blob:              Tisc_close_blob;
    constructor Create(ClientLibraryType: TClientLibrary; aLibraryPath: string = '');
    destructor Destroy; override;
    property ClientMajorVer: Byte read FClientMajorVersion;
    property ClientMinorVer: Byte read FClientMinorVersion;
    property ClientVersion: string read FClientVersion;
    property LibraryPath: string read GetLibraryPath write SetLibraryPath;
    property LibraryName: string read GetLibraryName;
    property Loaded: Boolean read LibraryLoaded;
  end;

var
  FBLoadedLibrariesCount: Byte;
  LoadedLibraries: array of TFIBClientLibrary;
threadvar
  FStatusVector : TStatusVector;

const
  ClientDLLs: array[0..3] of string = ('', 'FBEMBED', 'FBCLIENT', 'GDS32');


implementation

end.

