unit FIB_params;

interface
uses
  Windows, Types, Classes, SysUtils, Variants, Math, Dialogs,
  FmtBCD, SqlTimSt, DB,
  FIB_messages, FIB_header, FIB_array, FIB_blob, FIB_access,
  CommonStuff;

type

  TFIB_FieldsParams = class;

  TFIB_FieldParam = class(TObject)
  private
  protected
    function InternalGetMemoryStream: TMemoryStream; virtual; abstract;
    procedure SetPointer(Ptr: Pointer; Size: Integer);
    property Owner: TFIB_FieldsParams read FFieldsParams;
  public
    constructor Create(AOwner: TFIB_FieldsParams); virtual;
    destructor Destroy; override;
    procedure GetData(Buffer: Pointer); overload;
    procedure LoadFromFile(const FileName: AnsiString; BlobType: TBlobType);
    procedure LoadFromStream(Stream: TStream; BlobType: TBlobType);
    procedure SetBlobData(Buffer: Pointer; Size: Integer; BlobType: TBlobType = ftBlob);
    procedure SetData(Buffer: Pointer);
    property Data: Pointer read FData;
    property DataSize: Integer read FDataSize;
    property DataType: TFieldType read FDataType write SetDataType;
    property AsBCD: Currency read GetAsCurrency write SetAsCurrency;
    property AsFmtBCD: TBcd read GetAsFmtBCD write SetAsFmtBCD;
    property AsBlob: AnsiString{TBlobData} read GetAsString write SetAsBlob;
    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;
    property AsCurrency: Currency read GetAsCurrency write SetAsCurrency;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsDate: TDateTime read GetAsDateTime write SetAsDate;
    property AsTime: TDateTime read GetAsDateTime write SetAsTime;
    property AsFloat: Double read GetAsFloat write SetAsFloat;
    property AsInteger: LongInt read GetAsInteger write SetAsInteger;
    property AsLargeInt: Int64 read GetAsLargeInt write SetAsLargeInt;
    property AsSmallInt: LongInt read GetAsInteger write SetAsSmallInt;
    property AsWord: LongInt read GetAsInteger write SetAsWord;
    property AsGUID: TGuid read GetAsGUID write SetAsGUID;
    property AsSQLTimeStamp: TSQLTimeStamp read GetAsSQLTimeStamp write SetAsSQLTimeStamp;
    property AsMemo: AnsiString read GetAsString write SetAsString;
    property AsString: AnsiString read GetAsString write SetAsString;
    property AsWideString: WideString read GetAsWideString write SetAsWideString;
    property AsMemoryStream: TMemoryStream read GetAsMemoryStream;
    property AsQUAD: TGDS_QUAD read FGDS_QUAD write FGDS_QUAD;

    property ArrayElementSize: Word read FArrayElementSize
             write SetArrayElementSize;
    property ArrayBounds[Dimension: Byte]: TISC_ARRAY_BOUND
             read GetArrayBounds write SetArrayBounds;
    property ArrayElementValueByIndices[Indices: TSmallIntDynArray]: Variant
             read GetArrayElementValueByIndices
             write SetArrayElementValueByIndices;
    property ArrayElementValueByNumber[ElementNumber: Cardinal]: Variant
             read GetArrayElementValueByNumber
             write SetArrayElementValueByNumber;

    property IsNull: Boolean read GetIsNull;
    property Text: AnsiString read GetAsString write SetText;
    property Value: Variant read GetValue write SetValue;
    property Index: SmallInt read FIndex;
  end;

  TFIB_FieldsParams = class(TPersistent)
  private
  protected
    function InternalCreateItem: TFIB_FieldParam; virtual; abstract;
    function InternalAddItem(ItemsCount: Integer): TFIB_FieldParam;
    function InternalGetItem(Index1, Index2: SmallInt): TFIB_FieldParam;
    function GetItemsCount: Word; virtual; abstract;
    procedure InternalClear(FieldParamCount: SmallInt);
  public
    constructor Create(POwnerDB: PFIB_Database); virtual;
    property Count: Word read GetItemsCount;
  end;


  TFIB_Param = class(TFIB_FieldParam)
  private
  protected
    FParamType: TParamType;
    function InternalGetMemoryStream: TMemoryStream; override;
  public
    constructor Create(AOwner: TFIB_FieldsParams); override;
    property ParamType: TParamType read FParamType;
    property Name: AnsiString read GetName;
  end;

  TFIB_Params = class(TFIB_FieldsParams)
  private
  protected
    FSelStmt: string;
    function GetItemsCount: Word; override;
    function InternalCreateItem: TFIB_FieldParam; override;
    procedure AddParameter(ParamName: AnsiString; Value: Variant);
    procedure DelParameter;
    procedure Clear;
    procedure PrepareAll(TRHandle: TISC_TR_HANDLE; var PXSQLDA_IN: PXSQLDA);
  public
    constructor Create(POwnerDB: PFIB_Database); override;
    destructor Destroy; override;
    function FindParam(ParamName: string): TFIB_Param;
    function ParamByName(ParamName: string): TFIB_Param;
    property Items[Index: SmallInt]: TFIB_Param read GetItem; default;
    property ParamValues[ParamName: AnsiString]: Variant read GetParamValue write SetParamValue;
  published
    property SQLParams: TStrings read FSQLParams write SetSQLParams;
  end;


implementation

end.
